@@ -65,9 +65,9 @@ class GenericProgramMatch(deskbar.interfaces.Match):
             program = self._desktop.get_string("Exec")
             self.add_action( GenericAction(self.get_name(), program, self._args, verb) )
         else:
-            self.add_action( OpenDesktopFileAction(self.get_name(), self._desktop, self.desktop_file) )
+            self.add_action( OpenDesktopFileAction(self.get_name(), self._desktop, self.desktop_file, self._display_prog) )
 
-    def get_hash(self, text=None):
+    def get_hash(self):
         return "generic_"+self._display_prog
         
 class GnomeDictMatch(GenericProgramMatch):
@@ -109,6 +109,14 @@ class SpecialProgramHandler(deskbar.interfaces.Module):
         if self._match != None:
             self._match.set_priority( self.get_priority() + get_priority_for_name(qstring, self._match._desktop.get_string("Exec")) )
             self._emit_query_ready(qstring, [self._match] )
+            
+    @staticmethod
+    def desktop_file_exists(desktop):
+        for dir in get_xdg_data_dirs():
+            f = os.path.join(dir, "applications", desktop)
+            if os.path.exists(f):
+                return True
+        return False
         
 class GnomeDictHandler(SpecialProgramHandler):
     
@@ -126,7 +134,14 @@ class GnomeDictHandler(SpecialProgramHandler):
                     icon=desktop.get_string(deskbar.core.gnomedesktop.KEY_ICON),
                     desktop=desktop,
                     desktop_file=f)
-
+        
+    @staticmethod
+    def has_requirements():
+        if not SpecialProgramHandler.desktop_file_exists("gnome-dictionary.desktop"):
+            DevhelpHandler.INSTRUCTIONS = _("GNOME dictionary is not installed")
+            return False
+        return True
+        
 class GnomeSearchHandler(SpecialProgramHandler):
     
     INFOS = {'icon': deskbar.core.Utils.load_icon('system-search'),
@@ -144,6 +159,13 @@ class GnomeSearchHandler(SpecialProgramHandler):
                     desktop=desktop,
                     desktop_file=f)
         
+    @staticmethod
+    def has_requirements():
+        if not SpecialProgramHandler.desktop_file_exists("gnome-search-tool.desktop"):
+            DevhelpHandler.INSTRUCTIONS = _("GNOME search tool is not installed")
+            return False
+        return True
+        
 class DevhelpHandler(SpecialProgramHandler):
     
     INFOS = {'icon': deskbar.core.Utils.load_icon('devhelp'),
@@ -152,7 +174,10 @@ class DevhelpHandler(SpecialProgramHandler):
              'version': VERSION}
     
     def __init__(self):
-        SpecialProgramHandler.__init__(self, "devhelp.desktop")
+        if SpecialProgramHandler.desktop_file_exists("devhelp.desktop"):
+        	SpecialProgramHandler.__init__(self, "devhelp.desktop")
+        else:
+      		SpecialProgramHandler.__init__(self, "gnome-devhelp.desktop")
     
     def create_match(self, desktop, f):
         return DevhelpMatch(
@@ -160,6 +185,13 @@ class DevhelpHandler(SpecialProgramHandler):
                     icon=desktop.get_string(deskbar.core.gnomedesktop.KEY_ICON),
                     desktop=desktop,
                     desktop_file=f)
+        
+    @staticmethod
+    def has_requirements():
+        if not (SpecialProgramHandler.desktop_file_exists("devhelp.desktop") or SpecialProgramHandler.desktop_file_exists("gnome-devhelp.desktop")):
+            DevhelpHandler.INSTRUCTIONS = _("Devhelp is not installed")
+            return False
+        return True
 
 class OpenPathProgramAction(deskbar.interfaces.Action):
     
@@ -196,27 +228,30 @@ class OpenPathProgramAction(deskbar.interfaces.Action):
                 #No zenity, get out of the if, and launch without GUI
                 pass
         
-        spawn_async(text.split(" "))            
+        spawn_async(text.split(" "))
+        
+    def get_hash(self):
+        if self.use_terminal:
+            return self._name + "_terminal"
+        else:
+            return self._name
 
     def get_verb(self):
-        return _("Execute %s") % "<b>%(text)s</b>"
+        if self.use_terminal:
+            return _("Execute %s in terminal") % "<b>%(text)s</b>"
+        else:
+            return _("Execute %s") % "<b>%(text)s</b>"
 
 class PathProgramMatch(deskbar.interfaces.Match):
     
-    def __init__(self, name, command, use_terminal=False, priority=0, **args):
+    def __init__(self, name, command, priority=0, **args):
         deskbar.interfaces.Match.__init__(self, name=name, icon="gtk-execute", category="actions", **args)
-        self.use_terminal = use_terminal
         self.set_priority(self.get_priority() + EXACT_MATCH_PRIO)
-        self.add_action( OpenPathProgramAction(command, self.use_terminal) )
-        
-    def set_with_terminal(self, terminal):
-        self.use_terminal = terminal
+        self.add_action( OpenPathProgramAction(command, False), True )
+        self.add_action( OpenPathProgramAction(command, True) )
         
     def get_hash(self, text=None):
-        if not self.use_terminal:
-            return text
-        else:
-            return text+"_terminal"
+        return text
         
 class ProgramsHandler(deskbar.interfaces.Module):
     
@@ -251,16 +286,6 @@ class ProgramsHandler(deskbar.interfaces.Module):
             match._priority = get_priority_for_name(query, match._desktop.get_string("Exec"))
             result.append(match)
         return result
-            
-    
-    def on_key_press(self, query, shortcut):
-        if shortcut == gtk.keysyms.t:
-            match = self._check_program(query.split(" ")[0])
-            if match != None:
-                match.set_with_terminal(True)
-                return match
-            
-        return None    
                 
     def _scan_desktop_files(self):
         for dir in get_xdg_data_dirs():
